@@ -21,9 +21,6 @@ class AuthorizationResponse(message.Response):
     code = message.Parameter(str, required=True)
     state = message.Parameter(str, required=message.is_state_required)
 
-    def is_redirect(self):
-        return True
-
 
 class AuthorizationRequest(message.Request):
     response = AuthorizationResponse
@@ -36,30 +33,32 @@ class AuthorizationRequest(message.Request):
     state = message.Parameter(str, recommended=True)
 
     def answer(self, provider, owner):
+        client = provider.store.get_client(self.client_id)
+        if not isinstance(client, IClient)\
+                or not provider.authorize_client(client):
+            raise UnauthorizedClient(self)
+
+        redirect_uri = self.redirect_uri or client.get_redirect_uri()
+        if not redirect_uri:
+            raise InvalidRequest(self)
+        elif not provider.validate_redirect_uri(client, redirect_uri):
+            raise UnauthorizedClient(self, redirect_uri)
+
         try:
-            client = provider.store.get_client(self.client_id)
-            if not isinstance(client, IClient)\
-                    or not provider.authorize_client(client):
-                raise UnauthorizedClient()
-
-            redirect_uri = self.redirect_uri or client.get_redirect_uri()
-            if not redirect_uri:
-                raise InvalidRequest()
-            elif not provider.validate_redirect_uri(client, redirect_uri):
-                raise UnauthorizedClient()
-
             code = provider.store.issue_authorization_code(
                 client, owner, provider.normalize_scope(self.scope))
-            return self.response.from_dict(self, {
-                'code': code.get_code(),
-                'state': self.state,
-                'redirect_uri': redirect_uri,
-            })
         except ErrorException as why:
-            why.is_redirect = True
             why.request = self
             raise
 
+        response = self.response(self, redirect_uri)
+        response.update({
+            'code': code.get_code(),
+            'state': self.state,
+            'redirect_uri': redirect_uri,
+        })
+        return response
+
 
 class AccessTokenRequest(message.Request):
     response = message.AccessTokenResponse
@@ -72,31 +71,36 @@ class AccessTokenRequest(message.Request):
     client_id = message.Parameter(str, required=True)
 
     def answer(self, provider, owner):
+        authcode = provider.store.get_authorization_code(self.code)
+        if authcode is None or authcode.is_used():
+            # NOTES: If an authorization code is used more than once,
+            # the authorization server MUST deny the request and SHOULD
+            # revoke (when possible) all tokens previously issued
+            # based on that authorization code.
+            raise AccessDenied()
+
+        authcode.mark_as_used()
+
+        client = provider.store.get_client(self.client_id)
+        if not isinstance(client, IClient)\
+                or not provider.authorize_client(client)\
+                or client != authcode.get_client():
+            raise UnauthorizedClient()
+
         try:
-            authcode = provider.store.get_authorization_code(self.code)
-            if authcode is None or authcode.is_used():
-                # NOTES: If an authorization code is used more than once,
-                # the authorization server MUST deny the request and SHOULD
-                # revoke (when possible) all tokens previously issued
-                # based on that authorization code.
-                raise AccessDenied()
-
-            client = provider.store.get_client(self.client_id)
-            if not isinstance(client, IClient)\
-                    or not provider.authorize_client(client)\
-                    or client != authcode.get_client():
-                raise UnauthorizedClient()
-
             token = provider.store.issue_access_token(authcode.get_client(),
                                                       authcode.get_owner(),
                                                       authcode.get_scope())
-            return self.response.from_dict(self, {
-                'access_token': token.get_token(),
-                'token_type': token.get_type(),
-                'expires_in': token.get_expires_in(),
-                'refresh_token': token.get_refresh_token(),
-                'scope': ' '.join(token.get_scope()),
-            })
         except ErrorException as why:
             why.request = self
             raise
+
+        response = self.response(self)
+        response.update({
+            'access_token': token.get_token(),
+            'token_type': token.get_type(),
+            'expires_in': token.get_expires_in(),
+            'refresh_token': token.get_refresh_token(),
+            'scope': ' '.join(token.get_scope()),
+        })
+        return response
